function gain_fit,tel_bsl,funct,dt_smooth,x_var,y_var,baselines, $
                 distinct_baselines,xs,ys,wts,yfs
;
; Gain fit for amp and phase.
;
; The data on the gain calibrators is fit by baseline or telescope
;
; parameters : tel_bsl    -- type of fit ('telescope' or 'baseline')
;                            NOT USED AT ALL is this rountine, all
;                            we care here is fitting the amp and/or phase
;                            v.s. the x_var
;              dt_smooth  -- as in gain.pro, if dt_smooth > 0, boxcar
;                            smoothing is used and this is the smoothing
;                            interval size.
;                            if dt_smooth <=0 (NEGATIVE INTEGER), polynomical
;                            fitting is used, and this is the order of the
;                            polynomical
;                            if dt_smooth =-0.5, point-to-point connect
;                            will be used.
;              x_var      -- x-ccord :'int', 'hours', el'
;                            NOT USED AT ALL currently. All we do is
;                            fitting whatever xs vs the input y_var
;              y_var      -- y-coord : fit 'amp,pha','amp','pha','complex'
;              xs         -- x_coord's
;              ys         -- amp, pha, or complex
;              baselines  -- labeling of the data rx/band/bsl/sb combination
;              distinct_baselines -- 
;              wts        -- weight for each data point (neg. excluded
;                            from fit)
;              yfs        -- fitted curve
;
; result = -1 (error), 1 (succesful)
;
; To do a gain fit for amp w/ smoothing over 2 hours:  
; eg. : result=gai_fit(tel_bsl,2.,'hours','amp',4,xs,ys,yf)
;
common global
common data_set

if e.debug then begin
    print, 'Starting gain_fit with y_var of ',y_var
endif

n_singular=0
n_baselines=n_elements(distinct_baselines)
npts=n_elements(baselines)

for ib=0,n_baselines-1 do begin
   
  js=where(distinct_baselines(ib) eq baselines and wts gt 0.,count)
  js_bad=where(distinct_baselines(ib) eq baselines and wts le 0.,count_bad)

  if e.debug then print," Now doing baseline ", distinct_baselines(ib)

  ;
  ;  amp fit
  ;
  if y_var eq 'amp,pha' or y_var eq 'amp' then begin

    if e.debug then print, "fitting amplitude curve "

    if (funct eq 'connect') then begin

      if e.debug then print, "Connect mode, no fitting "

      yfs(0,js)=ys(0,js)
      if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 

    endif

    if (funct eq 'smooth') then begin

      if e.debug then print, "smooth with interval ", dt_smooth
      ;
      ; uti_interpate original data to smallest original spacing
      ;
      ;; Qi update min_dx to avoid very small min_dx 02/01/03
      ;;        min_dx=min(abs([x_ori[0:n_elements(x_ori)-2] $
      ;;                       -x_ori[1:n_elements(x_ori)-1]>0.]))
      x_ori=xs(js) & min_x=min([x_ori]) & max_x=max([x_ori]) 
      dx=abs(x_ori[0:n_elements(x_ori)-2]-x_ori[1:n_elements(x_ori)-1])
      j=where(dx gt 1.e-4)
      min_dx=min(dx(j))
      ;; EndOfChange
      
      ;print, x_ori
      ;print, min_x
      ;print, max_x
      ;print, "dx, min_dx", dx, min_dx

      nx_int=1+long((max_x-min_x)/min_dx)
      x_int=min_dx*findgen(nx_int)+min_x
      y_int=uti_interp(ys(0,js),x_ori,x_int)

      nx_int=1+long((max_x-min_x)/min_dx)
      ;print,"NX_INT", nx_int
      ;print, "X_INT", x_int
      ;print, "Y_INT", y_int

      ;
      ; then boxcar smooth it in time or integration #
      ;
      ;;        nf=long(min( [ nx_int/2,max([1,dt_smooth/(x_int[1]-x_int[0])]) ] ))

      nf=long(max([1,dt_smooth/(x_int[1]-x_int[0])]))
      ;print, x_int[1], x_int[0]
      ;print, x_int
      ;print, "NF ", nf

      if nf gt (nx_int-1) then nf=nx_int-1
      if (nf mod 2) ne 1 then nf=max([1,nf+1-2*(nf ge nx_int-1)])
      filter=make_array(nf,/float,value=(1./nf))
      y_conv=convol(y_int,filter,/edge_truncate)

      ii=fix(nf-1/2.)/2
      if ii eq 0 then begin
        print, 'dt_smooth value is too small for boxcar smoothing.'
        return,-1
      endif
      y_temp1=make_array(ii,/double,value=y_conv[ii])
      y_temp2=make_array(ii,/double,value=y_conv[nx_int-ii-1])
      y_temp=[y_temp1,y_conv[ii:nx_int-ii-1],y_temp2]
      y_smo=uti_interp(y_temp,x_int,x_ori)
      ;        y_smo=uti_interp(convol(y_int,filter,/edge_truncate),x_int,x_ori)
      yfs(0,js)=y_smo
      if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
    endif

    if (funct eq 'poly') then begin

      if e.debug then print, "polyfit with order ", -dt_smooth
      order=-dt_smooth
      y_temp = make_array(n_elements(js),/double)
      coeff = poly_fit(xs(js), ys(0,js),order,YFIT=y_temp,/double)
      if e.debug then print, "coeffs are ",coeff

      yfs(0,js)=y_temp
      if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 

    endif

  endif

  ;
  ;  pha fit
  ;
  if y_var eq 'amp,pha' or y_var eq 'pha' then begin

    if e.debug then print, "fitting phase curve "

    if (funct eq 'connect') then begin

      if e.debug then print, "Connect mode, no fitting "

      if y_var eq 'amp,pha' then begin
        yfs(1,js)=ys(1,js)
        if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE 
      endif
      if y_var eq 'pha' then begin
        yfs(0,js)=ys(0,js)
        if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
      endif

    endif

    if (funct eq 'smooth') then begin

      if e.debug then print, "smooth with interval ", dt_smooth
      ;
      ; uti_interpate original data to smallest original spacing
      ;
      x_ori=xs(js) & min_x=min([x_ori]) & max_x=max([x_ori]) 
      dx=abs(x_ori[0:n_elements(x_ori)-2]-x_ori[1:n_elements(x_ori)-1])
      j=where(dx gt 1.e-4)
      min_dx=min(dx(j))
      ;;        min_dx=min(abs([x_ori[0:n_elements(x_ori)-2] $
      ;;                       -x_ori[1:n_elements(x_ori)-1]]))
      nx_int=1+long((max_x-min_x)/min_dx)
      x_int=min_dx*findgen(nx_int)+min_x
      if y_var eq 'amp,pha' then y_int=uti_interp(ys(1,js),x_ori,x_int)
      if y_var eq 'pha' then y_int=uti_interp(ys(0,js),x_ori,x_int)

      ;
      ; then boxcar smooth it in time or integration #
      ;
      ;;        nf=long(min( [ nx_int/2,max([1,dt_smooth/(x_int[1]-x_int[0])]) ] ))

      nf=long(max([1,dt_smooth/(x_int[1]-x_int[0])]))
      if nf gt (nx_int-1) then nf=nx_int-1
      if (nf mod 2) ne 1 then nf=max([1,nf+1-2*(nf ge nx_int-1)])
      filter=make_array(nf,/float,value=(1./nf))
      y_conv=convol(y_int,filter,/edge_truncate)

      ii=fix(nf-1/2.)/2
      if ii eq 0 then begin
        print, 'dt_smooth value is too small for boxcar smoothing.'
        return,-1
      endif
      y_temp1=make_array(ii,/double,value=y_conv[ii])
      y_temp2=make_array(ii,/double,value=y_conv[nx_int-ii-1])
      y_temp=[y_temp1,y_conv[ii:nx_int-ii-1],y_temp2]
      y_smo=uti_interp(y_temp,x_int,x_ori)
      ;;        y_smo=uti_interp(convol(y_int,filter,/edge_truncate),x_int,x_ori)

      if y_var eq 'amp,pha' then begin
        yfs(1,js)=y_smo
        if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE 
      endif
      if y_var eq 'pha' then begin
        yfs(0,js)=y_smo
        if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
      endif

    endif

    if (funct eq 'poly') then begin

      if e.debug then print, "polyfit with order ", -dt_smooth

      order=-dt_smooth
      y_temp = make_array(n_elements(js),/double)

      if y_var eq 'amp,pha' then begin
        coeff = poly_fit(xs(js), ys(1,js),order,YFIT=y_temp,/double)
        if e.debug then print, "coeffs are ",coeff

        yfs(1,js)=y_temp
        if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE 
      endif
      if y_var eq 'pha' then begin
        coeff = poly_fit(xs(js), ys(0,js),order,YFIT=y_temp,/double)
        if e.debug then print, "coeffs are ",coeff

        yfs(0,js)=y_temp
        if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
      endif

    endif

  endif

  if y_var eq 'complex' then begin
    
    if e.debug then print, "fitting complex curve "

    uti_conv_apc,cmp,reform(ys(0,js),count),reform(ys(1,js),count), $
                /complex


    if (funct eq 'connect') then begin

      if e.debug then print, "Connect mode, no fitting "

      print, 'Not implemented correctly yet --- '
      print, 'Treatment for the complex-amp/pha conversion needs to be done.'
      return, -1

;      if y_var eq 'amp,pha' then begin
;        yfs(1,js)=ys(1,js)
;        if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE 
;      endif
;      if y_var eq 'pha' then begin
;        yfs(0,js)=ys(0,js)
;        if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
;      endif

    endif

    if (funct eq 'smooth') then begin

      if e.debug then print, "smooth with interval ", dt_smooth
      ;
      ; uti_interpate original data to smallest original spacing
      ;
      x_ori=xs(js) & min_x=min([x_ori]) & max_x=max([x_ori]) 
      x_ori=double(xs(js)) & min_x=min([x_ori]) & max_x=max([x_ori]) 
      dx=abs(x_ori[0:n_elements(x_ori)-2]-x_ori[1:n_elements(x_ori)-1])
      j=where(dx gt 1.e-4)
      min_dx=min(dx(j))
      ;;     min_dx=min(abs([x_ori[0:n_elements(x_ori)-2] $
      ;;                     -x_ori[1:n_elements(x_ori)-1]]))
      nx_int=1+(max_x-min_x)/min_dx
      x_int=min_dx*findgen(nx_int)+min_x
      real_int=uti_interp(float(cmp),x_ori,x_int)
      imag_int=uti_interp(imaginary(cmp),x_ori,x_int)

      ;
      ; then boxcar smooth it in time or integration #
      ;

;      nf=long(min( [ nx_int/2,max([1,dt_smooth/(x_int[1]-x_int[0])]) ] ))
      nf=long(max([1,dt_smooth/(x_int[1]-x_int[0])]))
      if (nf mod 2) ne 1 then nf = max([1,nf+1-2*(nf ge nx_int-1)])
      filter=make_array(nf,/float,value=(1./nf))
      real_smo=uti_interp(convol(real_int,filter,/edge_truncate),x_int,x_ori)
      imag_smo=uti_interp(convol(imag_int,filter,/edge_truncate),x_int,x_ori)
      uti_conv_apc,complex(real_smo,imag_smo),amp_smo,pha_smo, $
                   /amp_pha
      result=uti_pha_unwrap(pha_smo)
      yfs(0,js)=amp_smo & yfs(1,js)=pha_smo
      if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE
      if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE
    endif

    if (funct eq 'poly') then begin

      print, 'Not implemented correctly yet --- '
      print, 'poly_fit.pro does not work on complex array directly.'
      return, -1

;      if e.debug then print, "polyfit with order ", -dt_smooth
;      order=-dt_smooth
;      y_temp = make_array(n_elements(js),/complex)
;      coeff = poly_fit(xs(js), cmp(0,js),order,YFIT=y_temp,/double)
;      if e.debug then print, "coeffs are ",coeff
;
;      uti_conv_apc,y_temp,amp_temp,pha_temp, $
;                   /amp_pha
;      yfs(0,js)=amp_temp
;      yfs(1,js)=pha_temp
;      if count_bad gt 0 then yfs(0,js_bad)=!BAD_VALUE 
;      if count_bad gt 0 then yfs(1,js_bad)=!BAD_VALUE 
;
    endif

  endif

endfor

return,1

end
