pro select,source=source,baseline=baseline,band=band,integration=integration, $
         sideband=sideband,reset=reset,display=display,pos_wt=pos_wt
;yes
;=Task:SELECT --- To select subsets of data (SMA wrapper)
;#Type: utility
;+Use:
;      SELECT is used to select the subsets of the data. When called with 
;      no arguments, the tasks will display the sources, baselines, etc.
;      of the current data without taking any further action. Select may
;      be run more than once to select a subset of previously selected data.
;      One can use the parameter /reset to clear any previous selections,
;      and use the parameter /pos_wt to select data with positive wights.
;      Selection keywords include source, baseline, band, integration,
;      sideband, and selection flag include reset, display, and pos_wt.
;      The following are some examples:
;      >select,source='jupiter',/reset
;      >select,source=['jupiter','3c273'],/reset
;      >select,baseline=['1-2','1-3','1-4']
;      >select,integration=[1,10,100,110,200,210],sideband='u'
;      >select,/pos_wt
;      >select,source='jupiter',sideband='u', /pos_wt
;
;
;&history:
;------------------------------------------------------------------------
;      cykuo 26feb04 adapting the header
;------------------------------------------------------------------------

common global
common data_set
common plo

printall = 0
if keyword_set(source) eq 0 and keyword_set(baseline) eq 0 $
   and  keyword_set(band) eq 0 and keyword_set(sideband) eq 0  $
   and keyword_set(integration) eq 0 and keyword_set(pos_wt) $
   eq 0 then printall = 1
if keyword_set(display)  then printall = 1

if printall then begin
print,'All Sources: ',c.source
print,'All Baselines: ',c.blcd
print,'All Bands: ',c.band
print,'All Sidebands : ',c.sb
print,'All Integrations: ',strtrim(string(in[0].int),2),'-', $
  strtrim(string(in[n_elements(in)-1].int),2)
endif


if (keyword_set(reset)) then result=dat_filter(s_f,/reset,/no_notify)
if (keyword_set(pos_wt)) then result=dat_filter(s_f,'"wt" gt "0"',/no_notify)

list=''

if (keyword_set(source)) then begin

if (source[0] ne '') then begin
    list=list+' ("source" eq "'+strtrim(source[0],2)+'"'
  if n_elements(source) gt 1 then begin
    for i = 1, n_elements(source)-1 do begin
        list=list+' or "source" eq "'+strtrim(source[i],2)+'"'
    endfor
  endif
  list = list + ')'
endif

list = list + ' and '
endif

if (keyword_set(baseline)) then begin


if (baseline[0] ne '') then begin
  list=list+' ("blcd" eq "'+strtrim(baseline[0],2)+'"'
   if n_elements(baseline) gt 1 then begin
      for i = 1, n_elements(baseline)-1 do begin
            list=list+' or  "blcd" eq "'+strtrim(baseline[i],2)+'"'
      endfor
   endif
  list = list + ')'
endif                                                                                  

list = list + ' and '
endif

if (keyword_set(band)) then begin

if (band[0] ne '') then begin
  list=list+' ("band" eq "'+strtrim(band[0],2)+'"'
  if n_elements(band) gt 1 then begin
    for i = 1, n_elements(band)-1 do begin
        list=list+' or "band" eq "'+strtrim(band[i],2)+'"'
    endfor
  endif
  list = list + ')'
endif

list = list + ' and '
endif

if (keyword_set(sideband)) then begin

sb = sideband

if (sb[0] ne '') then begin
  list=list+' ("sb" eq "'+strtrim(sb[0],2)+'"'
   if n_elements(sb) gt 1 then begin
      for i = 1, n_elements(sb)-1 do begin
            list=list+' or  "sb" eq "'+strtrim(sb[i],2)+'"'
      endfor
   endif
  list = list + ')'
endif 

list = list + ' and '
endif

if (keyword_set(integration)) then begin

int = string(integration)

if (int[0] ne '') then begin
  list=list+'(  ("int" ge "'+strtrim(int[0],2)+'"'
  list=list+' and  "int" le "'+strtrim(int[1],2)+'")'
   if n_elements(int) gt 2 then begin
      for i = 2, n_elements(int)-1,2 do begin
            list=list+' or ("int" ge "'+strtrim(int[i],2)+'"'
            list=list+' and  "int" le "'+strtrim(int[i+1],2)+'")'
      endfor
   endif
  list = list + ')'
endif              
list = list + ' and '
endif

if list ne '' then begin
  nch = strlen(list)
  list = strmid(list,0,nch-5)
  print,'Here are the data selection criteria '
  print,list
  npts=dat_filter(s_f,list)
endif

if (keyword_set(display)) then begin

junk = in[pif].isource
distinct_i=junk( uniq(junk,sort(junk)) )
n = n_elements(distinct_i)
print,'Selected sources:  ',c.source(distinct_i)

junk = bl[pbf].iblcd
distinct_i=junk( uniq(junk,sort(junk)) )
n = n_elements(distinct_i)
print,'Selected baselines:  ',c.blcd(distinct_i)


junk = sp[psf].iband
distinct_i=junk( uniq(junk,sort(junk)) )
n = n_elements(distinct_i)
print,'Selected bands:  ',c.band(distinct_i)

junk = bl[pbf].isb
distinct_i=junk( uniq(junk,sort(junk)) )
n = n_elements(distinct_i)
print,'Selected sidebands:  ',c.sb(distinct_i)

junk = in[pif].int
distinct_i=junk( uniq(junk,sort(junk)) )
n = n_elements(distinct_i)
listints = strtrim(string(distinct_i[0]),2)
for i = 1,n-2 do begin
 if (distinct_i[i]+1 ne distinct_i[i+1]) then begin
   listints = listints + '-' + strtrim(string(distinct_i[i]),2) $
     + ', '+strtrim(string(distinct_i[i+1]),2)
 endif
endfor
listints = listints + '-'+strtrim(string(distinct_i[n-1]),2)
print,'Selected integrations: ',listints
endif

end

